@@ -1,5 +1,5 @@
 /* flac_ren - renamer part of utility to add FLAC support to Monkey's Audio
- * Copyright (C) 2000,2001,2002,2003,2004,2005  Josh Coalson
+ * Copyright (C) 2000,2001,2002,2003,2004,2005,2006  Josh Coalson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,10 +16,14 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include<io.h>
-#include<sys/stat.h>
-#include<wtypes.h>
-#include<winbase.h>
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#include <io.h>
+#include <sys/stat.h>
+#include <wtypes.h>
+#include <winbase.h>
 
 int main(int argc, char *argv[])
 {
